/*
 * PjViewer.java
 *
 * Created on December 9, 2004, 6:49 PM
 */

package Grapher.Viewer;

import java.util.Hashtable;
import java.util.Enumeration;
import jv.anim.PsAnimation;
import jv.anim.PsTimeEvent;
import java.awt.Color;
import jv.function.PuFunction;
import jv.object.PsDebug;
import jv.project.PjProject;
import jv.vecmath.PdVector;
import jv.project.PvDisplayIf;
import jv.project.PvCameraIf;
import jv.project.PvViewerIf;
import jv.thirdParty.ruler.*;
import jv.number.PuDouble;
import Grapher.geom.*;
import jv.project.PgGeometry;
import jv.vecmath.PdMatrix;
import jv.geom.PgPointSet;
import Grapher.util.PuDomain;


/**
 * Main project file for displaying multiple geometries
 * 
 * @author		David Lippman
 * @version             1.0.281005
 */
public class PjViewer extends PjProject {
        /**f(x,y) type*/
        public final static int TYPE_FUNC = 1; 
        /**Parametric Surface type*/
        public final static int TYPE_PSURF = 2;  
        /**Parametric Curve type*/
        public final static int TYPE_PCURVE = 3; 
        /**Implicit Surface type*/
        public final static int TYPE_IMPSURF= 4; 
        /**Vector Field on grid type*/
        public final static int TYPE_VF= 5; 
        /**Vector Field on surface type*/
        public final static int TYPE_VFOS = 6; 
        /**Implicit Cylindrical surface type*/
        public final static int TYPE_IMPCYL = 7; 
        /**Implicit Spherical surface type*/
        public final static int TYPE_IMPSPH = 8; 
        
        protected       Hashtable               m_graphs;
        protected       PuDomain                m_clipbounds;
//        protected       PuDouble                m_bClip;
       
        protected       int                     m_gDef = 1;
        protected       double                  time = 2.0;
        protected       PsAnimation             anim;
        
	public PjViewer() {
		super("Parametrized Surface");

                m_graphs = new Hashtable();

           	PsAnimation anim = new PsAnimation();	// Create an animation, which sends timer events
		anim.setName("Associate Family");	// Set title of the animation dialog.
		anim.addTimeListener(this);			// Register this project in animation to receive setTime(time) events.

		if (getClass() == PjViewer.class)
			init();
	}
        
        /** Initializes the global clipbounds and initializes the project */
	public void init() {
                super.init();
                
                m_clipbounds = new PuDomain();
                m_clipbounds.setVars(new String[] {"x","y","z"});
                m_clipbounds.setDefBnds(new double[] {-1000.0,1000.0,1.,5.});
                m_clipbounds.setDefVals(new double[] {-5.,5.});
                m_clipbounds.setNumDiscr(0);
                m_clipbounds.init();
                m_clipbounds.setParent(this);
              
		
 
	}
        
        /**Starts the project and sets the clipbounds*/
	public void start() {
            	if (super.hasAnimation()) {
			anim = super.getAnimation();
			anim.setTimeInterval(-3., 3., .1, 1);
        //		anim.getAnimationPanel().setVisible(true);	// Show anmiation panel
	//		anim.start();											// Start animation
		}

                super.start();	
                // Parse parameters from Html page or command line, if available
		/* Axes do not turn on properly this way -- TO DO: FIX
                PvViewerIf viewer = getViewer();
		if (viewer != null) {
			String AxesOn = viewer.getParameter("Axes");
			if (AxesOn.equalsIgnoreCase("ON")) {
                            PvDisplayIf disp = getDisplay();
                            disp.showAxes(true);
                            disp.showBndBox(true);
                            ((jv.viewer.PvDisplay)disp).getAxes().setMode(PgAxes.AXES_3DBOUNDINGBOX);
                            ((jv.viewer.PvDisplay)disp).getAxes().update( ((jv.viewer.PvDisplay)disp).getAxes());
                        }
                }        
                */
                
                PvDisplayIf disp = getDisplay();
//                disp.showFrame(true);
                disp.setBackgroundColor(Color.white);
                
                for (int i=0; i<3;i++) {
                    disp.setClipBounds(i, new double[] {m_clipbounds.getMin(i), m_clipbounds.getMax(i)});
                }
              
		
	}

	/**
	 * Update the class whenever a child has changed.
	 * Method is usually invoked from the children.
	 */
	public boolean update(Object event) {
                if (event == this){
                    
                    return super.update(this);
                } else if (event == m_clipbounds) {
                    updateClip();
                    return true;
                }
                
		return super.update(event);
	}    
        
        /** Adds a graph to the display
         *  @param  graphname   The name used to reference the graph
         *  @param  type        The graph type TYPE_*
         *  @return true if the graph was added
         */
        public boolean addGraph(String graphname, int type) {

        switch (type) {
            case TYPE_FUNC:
                PgGraph newfunc = new PgGraph(graphname);
                newfunc.setParent(this);
                m_graphs.put(graphname,newfunc);
                addGeometry(newfunc);
                selectGeometry(newfunc);
                m_gDef++;
                return true;
            case TYPE_PSURF:
                PgPSurf newpsurf = new PgPSurf(graphname);
                newpsurf.setParent(this);
                m_graphs.put(graphname,newpsurf);
                addGeometry(newpsurf);
                selectGeometry(newpsurf);
                m_gDef++;
                return true;
            case TYPE_PCURVE:
                PgPCurve newpcurve = new PgPCurve(graphname);
                newpcurve.setParent(this);
                m_graphs.put(graphname,newpcurve);
                addGeometry(newpcurve);
                selectGeometry(newpcurve);
                m_gDef++;
                return true;
            case TYPE_IMPSURF:
                PgImpSurf newimpsurf = new PgImpSurf(graphname,TYPE_IMPSURF);
                newimpsurf.setParent(this);
                m_graphs.put(graphname,newimpsurf);
                addGeometry(newimpsurf);
                selectGeometry(newimpsurf);
                m_gDef++;
                return true;
            case TYPE_IMPCYL:
                PgImpSurf newimpcyl = new PgImpSurf(graphname,TYPE_IMPCYL);
                newimpcyl.setParent(this);
                m_graphs.put(graphname,newimpcyl);
                addGeometry(newimpcyl);
                selectGeometry(newimpcyl);
                m_gDef++;
                return true;
            case TYPE_IMPSPH:
                PgImpSurf newimpsph = new PgImpSurf(graphname,TYPE_IMPSPH);
                newimpsph.setParent(this);
                m_graphs.put(graphname,newimpsph);
                addGeometry(newimpsph);
                selectGeometry(newimpsph);
                m_gDef++;
                return true;
            case TYPE_VF:
                PgVecFld newvf = new PgVecFld(graphname);
                newvf.setParent(this);
                m_graphs.put(graphname, newvf);
                addGeometry(newvf);
                selectGeometry(newvf);
                m_gDef++;
                return true;
            case TYPE_VFOS:
                PgVecFldOn newvfon = new PgVecFldOn("VF on " + graphname);
                newvfon.setParent(this);
                PgPointSet base = (PgPointSet) m_graphs.get(graphname);
                newvfon.setGeometry(base);
                base.addVectorField(newvfon);
                //newvfon.update(newvfon);
                anim.addTimeListener(newvfon);
                newvfon.computeVfos();
                m_graphs.put("VF on "+graphname,newvfon);
                return true;
        }
                
        return false;
                
            
        }
/*        
        public boolean addVf(String vfbasename) {

                String graphname = "VF on " + vfbasename;
                

                m_graphs.put(graphname, (PjMulti) m_graphs.get(vfbasename));
 		PjMulti vfgraph = (PjMulti)m_graphs.get(vfbasename);
                if (vfgraph.addVf()) {
                     return true;
                } else {return false;}
        }
 */     
        
        /**
         * Removes a graph from the display
         * @param   gname   The name used to reference the graph
         * @return  true if the graph has a vectorfield attached, otherwise false
         */
        public boolean removeGraph(String gname) {
                boolean hasvf = false;
                
                
                if (gname.startsWith("VF on ")) {   
                    String basename = gname.substring(6);
                    PgPointSet newgraph = (PgPointSet) m_graphs.get(basename);
                    newgraph.removeAllVectorFields();
                    newgraph.update(newgraph);
                } else {
                    PgPointSet newgraph = (PgPointSet) m_graphs.get(gname);
                    removeGeometry(newgraph);
                }
                
                if (m_graphs.containsKey("VF on " + gname)) {
                    m_graphs.remove("VF on " + gname);
                    hasvf = true;
                }
                m_graphs.remove(gname);
                return hasvf;
        }
        
        /** 
         * Sets BoxRatio to scale the axes 
         * @see PvDisplayIf
         */
        public void setScl() {
                PvDisplayIf disp = getDisplay();
                disp.setEnabledBoxRatio(true);
         
                
                double xsize = m_clipbounds.getMax(0) - m_clipbounds.getMin(0);
                double ysize = m_clipbounds.getMax(1) - m_clipbounds.getMin(1);
                double zsize = m_clipbounds.getMax(2) - m_clipbounds.getMin(2);
                double scl = Math.min(xsize,Math.min(ysize,zsize));
    //            double scl = ysize;
                
                PdVector br = new PdVector(scl/xsize,scl/ysize,scl/zsize);
 //               br.normalize();
                 disp.setBoxRatio(br);
                
 //               disp.setBoxRatio(new PdVector(xscl.getValue(),yscl.getValue(),zscl.getValue()));
                update(this);
                 
                disp.update(disp);
        //        fitDisplays();
                
        }
        
        /**
         * Turns off BoxRatio to scale axes 
         */
        public void unsetScl() {
            PvDisplayIf disp = getDisplay();
                disp.setEnabledBoxRatio(false);
                
                disp.update(disp);
        }
        
        /**
         * Sets the camera view to perspective or another projection
         */
        public void setCamView(int view) {
            PvDisplayIf disp = getDisplay();
            disp.selectCamera(view);
            if (disp.isShowingAxes()) {
                ((jv.viewer.PvDisplay)disp).getAxes().update( ((jv.viewer.PvDisplay)disp).getAxes());
            }
            fitDisplays();
            disp.update(disp);
          
            
        }
        
        /** 
         * Centers the display on the the clipbounds and zooms to fit.  
         * If BoxRatio is used (setScl has been called), view is set to top view as a hack to get around
         * boxratio not being implemented properly
         * Removed 1/22/05 - updated JavaView fixed the problem
         */
  /*      public void setCamCenter() {
            PvDisplayIf disp = getDisplay();
            PvCameraIf cam = disp.getCamera();
               
            double theta = cam.getFieldOfView();
                double xsize = m_clipbounds.getMax(0) - m_clipbounds.getMin(0);
                double ysize = m_clipbounds.getMax(1) - m_clipbounds.getMin(1);
                double zsize = m_clipbounds.getMax(2) - m_clipbounds.getMin(2);
            
            double dist;
            if (disp.isEnabledBoxRatio()) {
                PdVector interest = new PdVector(m_clipbounds.getMin(0)+.5*xsize,m_clipbounds.getMin(1)+.5*ysize,m_clipbounds.getMin(2)+.5*zsize);
                
                
                cam.setViewDir(new PdVector(0,0,-1));
                cam.setUpVector(new PdVector(0,1,0));
                
                cam.setInterest(interest);
                double scl = Math.min(xsize,Math.min(ysize,zsize));
                dist = 0.8*scl/Math.tan(theta);
                cam.setDist(dist);

            } else {
                PdVector interest = new PdVector(m_clipbounds.getMin(0)+.5*xsize,m_clipbounds.getMin(1)+.5*ysize,m_clipbounds.getMin(2)+.5*zsize);
 
                double scl = Math.max(xsize,Math.max(ysize,zsize));
                dist = 0.6*scl/Math.tan(theta);
               
               cam.setInterest(interest);
               cam.setDist(dist);
            }
             
            disp.update(cam);
               
        }
    */    
        /** Updates the clipbounds.  Method is called when the clipbound values are changed */
        public void updateClip() {
            PvDisplayIf disp = getDisplay();
            for (int i=0; i<3;i++) {
                    disp.setClipBounds(i, new double[] {m_clipbounds.getMin(i), m_clipbounds.getMax(i)});
            }
            if (disp.isEnabledBoxRatio()){
                setScl();
            }
            
            fitDisplays();
            disp.update(disp);
       }
        
        /** Turns on or off clipping of the display using the clipbounds
         * @param   turnon  True to turn on clipping, false to turn it off
         */
        public void setClip(boolean turnon) {
            if (turnon) {
                PvDisplayIf disp = getDisplay();
                disp.setEnabledClipBounds(true);
                fitDisplays();
                disp.update(disp);
            } else {
                PvDisplayIf disp = getDisplay();
                disp.setEnabledClipBounds(false);
                fitDisplays();
                disp.update(disp);
            }
        }
        
        	/**
	 * Method is called from animation(-panel) whenever time has changed
	 * since this project is registered as dynamic object in the animation.
	 * Method is optional, but allows a project to update geometry to new current time.
	 */
	public boolean setTime(PsTimeEvent timeEvent) {
		time = timeEvent.getTime();
                for ( Enumeration e = m_graphs.keys() ; e.hasMoreElements() ; )
                {
                        PgGeometry newgraph =  (PgGeometry) m_graphs.get(e.nextElement());
                        newgraph.update(newgraph);      
                }
		return super.update(this);				// Update info panel of this project
	}

        public double getTime() {
            return time;
        }
    
}

