/*
 * PjViewer_IP.java
 *
 * @author		David Lippman
 * @version             1.0.270105
 */

package Grapher.Viewer;

import java.awt.event.*;
import java.awt.*;
import java.awt.datatransfer.*;

import java.lang.SecurityManager;
import jv.objectGui.PsTabPanel;
import jv.object.PsConfig;
import jv.object.PsDebug;
import jv.object.PsObject;
import jv.objectGui.PsSlotLayout;
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PvDisplayIf;
import jv.project.PgGeometryIf;
import jv.project.PjProject_IP;
import jv.anim.PsAnimation_IP;
import Grapher.geom.*;
import Grapher.util.*;
import jv.geom.PgPointSet;
import jv.project.PgGeometry;
import jv.thirdParty.ruler.*;

/**
 * Info panel for PjViewer 
 * 
 * @author		David Lippman
 * @version		1.0.122204
 */
public class PjViewer_IP extends PjProject_IP implements ItemListener,ActionListener,FocusListener {
        /**f(x,y) type*/
        public final static int TYPE_FUNC = 1; 
        /**Parametric Surface type*/
        public final static int TYPE_PSURF = 2;  
        /**Parametric Curve type*/
        public final static int TYPE_PCURVE = 3; 
        /**Implicit Surface type*/
        public final static int TYPE_IMPSURF= 4; 
        /**Vector Field on grid type*/
        public final static int TYPE_VF= 5; 
        /**Vector Field on surface type*/
        public final static int TYPE_VFOS = 6; 
        /**Implicit Cylindrical surface type*/
        public final static int TYPE_IMPCYL = 7; 
        /**Implicit Spherical surface type*/
        public final static int TYPE_IMPSPH = 8;    
        
        protected	PjViewer			m_PjViewer;
        protected       Choice                          m_selector;
        protected       Choice                          m_cNew;
        protected       Checkbox                        m_cClip;
        protected       Checkbox                        m_cShowFrame;
        protected       Checkbox                        m_cShowAxes;
        protected       TextField                       m_tmin;
        protected       TextField                       m_tmax;
        protected       Button                          m_bAnimstart;
        protected       boolean                         m_bCanUseClipboard = false;
        
        protected       PsTabPanel                      m_tabpanel;
        protected       PsPanel                         m_pMain;
        protected       PsPanel                         m_pSel;
        protected       PsPanel                         m_pAnim;
        protected       PsPanel                         cps;
        protected       PsPanel                         cp;
        protected       PsPanel                         m_pBnds;
        protected       Button                          m_bZoomFit;
        protected       Button                          m_bRemove;
        protected       Button                          m_bToClipboard;
        
        protected       PsPanel                         m_pView;
        protected       CheckboxGroup                   m_ViewGroup;
        protected       Checkbox[]                      m_cViews;
        protected       Button                          m_bGetClip;
        protected       Checkbox                         m_bSclAxes;
        protected       String[]                        m_typenames = {"Choose a graph to add",
                                                                        "Parametric Curve",
                                                                        "Parametric Surface",
                                                                        "z=f(x,y) Surface",
                                                                        "Implicit Surface",
                                                                        "Implicit Cylindrical",
                                                                        "Implicit Spherical",
                                                                        "Vector Field",
                                                                        "Vector Field on Selected"};
       protected        int[]                           m_typetable = {0, 
                                                                        TYPE_PCURVE, 
                                                                        TYPE_PSURF,
                                                                        TYPE_FUNC,
                                                                        TYPE_IMPSURF,
                                                                        TYPE_IMPCYL,
                                                                        TYPE_IMPSPH,
                                                                        TYPE_VF,
                                                                        TYPE_VFOS};
       protected        String[]                        m_typeshortnames = {
                            "None","PCurve", "PSurf", "Func", "ImpSurf", "ImpCyl","ImpSph","VF","VF on"};            
       
        /** Constructor.  Calls init() */
	public PjViewer_IP() {
		super();
		if (getClass() == PjViewer_IP.class)
			init();
	}
        
        /** Initializes the infopanel.  Creates the tabpanel and selectors */
	public void init() {
                super.init();
                
                m_tabpanel = new PsTabPanel();
                add(m_tabpanel);
                m_tabpanel.init();
                
               
                m_pMain = new PsPanel();
                
 //               m_pSel = new PsPanel(new GridLayout(2, 2));
                PsPanel m_pSel1 = new PsPanel (new PsSlotLayout(3));
                m_pSel1.add("1", new Label("New graph:"));
                m_cNew = new Choice();
                m_cNew.addItemListener(this);
  
                for (int i = 0; i< m_typenames.length; i++) {
                    m_cNew.add(m_typenames[i]);
                }
/*                m_cNew.add("Choose a graph to add");
                m_cNew.add("Parametric Curve");
                m_cNew.add("Parametric Surface");
                m_cNew.add("z=f(x,y) Surface");
                m_cNew.add("Vector Field");
                m_cNew.add("Vector Field on selected");
                m_cNew.add("Implicit Surface");
*/                                
                m_pSel1.add("2", m_cNew);
                m_pMain.add(m_pSel1);
                
                PsPanel m_pSel2 = new PsPanel (new PsSlotLayout(3));
                m_pSel2.add("1", new Label("Select Existing:"));
                m_selector = new Choice();
                m_selector.addItemListener(this);
                m_pSel2.add("2", m_selector);
                m_pMain.add(m_pSel2);
                
             
                cps = new PsPanel(new GridLayout(1, 1));
                cps.setName("cps");
                cps.setVisible(true);
                cps.setBorderType(PsPanel.BORDER_NONE);
                m_pMain.add(cps);
                
                m_pBnds = new PsPanel();
                
                
                m_pAnim = new PsPanel(new GridLayout(1,5));
                m_pAnim.add(new Label("Time:") );
                m_tmin = new TextField("-3",5);
                m_tmin.addActionListener(this);
                m_tmin.addFocusListener(this);
                m_pAnim.add(m_tmin);
                m_pAnim.add(new Label("to") );
                m_tmax = new TextField("3",5);
                m_tmax.addActionListener(this);
                m_tmax.addFocusListener(this);
                m_pAnim.add(m_tmax);
                m_bAnimstart = new Button ("Start");
                m_bAnimstart.addActionListener(this);
                m_pAnim.add(m_bAnimstart);
                m_pMain.add(m_pAnim);
                
                m_bZoomFit = new Button ("Zoom Fit");
                m_bZoomFit.addActionListener(this);
                
                m_bRemove = new Button ("Remove Selected");
                m_bRemove.addActionListener(this);
               
                
                try {
                  
                    SecurityManager sm = System.getSecurityManager();
                    if (sm != null) sm.checkSystemClipboardAccess();
                    
                    m_bCanUseClipboard = true;
                    m_bToClipboard = new Button ("Copy to Clipboard");
                    m_bToClipboard.addActionListener(this);
                    
                }
                catch (SecurityException e) {
                     
                   }
                
                PsPanel pBottomButtons = new PsPanel();
                if (m_bCanUseClipboard) {
                    pBottomButtons.setLayout(new GridLayout(2,2));
                    pBottomButtons.add(m_bZoomFit);
                    pBottomButtons.add(m_bRemove);
                    pBottomButtons.add(m_bToClipboard);
                } else {
                    pBottomButtons.setLayout(new GridLayout(1,2));
                    pBottomButtons.add(m_bZoomFit);
                    pBottomButtons.add(m_bRemove);
 
                }
		m_pMain.add(pBottomButtons);
		
		
		addTitle("");
	}
        
        /** Sets the infopanel's parent, and adds checkboxes and buttons to the panel */
	public void setParent(PsUpdateIf parent) {
		super.setParent(parent);
		m_PjViewer = (PjViewer)parent;
                
 //               m_pBnds.add(m_PjViewer.xscl.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
 //              m_pBnds.add(m_PjViewer.yscl.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
 //               m_pBnds.add(m_PjViewer.zscl.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
                
                m_cClip = new Checkbox("Use Clip Bounds");
                m_cClip.addItemListener(this);
                m_pBnds.add(m_cClip);
                
                m_cShowFrame = new Checkbox("Show XYZ Frame");
                m_cShowFrame.addItemListener(this);
                m_pBnds.add(m_cShowFrame);
                
                m_cShowAxes = new Checkbox("Show BoundBox and axes");
                PvDisplayIf disp = m_PjViewer.getDisplay();
                m_cShowAxes.setState(disp.isShowingAxes());
                m_cShowAxes.addItemListener(this);
                m_pBnds.add(m_cShowAxes);

                m_pView = new PsPanel(new GridLayout(1,5));
                m_pView.add(new Label("View:"));
                m_ViewGroup = new CheckboxGroup();
                m_cViews = new Checkbox[4];
                m_cViews[0] = new Checkbox("3D",m_ViewGroup,true);
                m_cViews[1] = new Checkbox("XY",m_ViewGroup,false);
                m_cViews[2] = new Checkbox("XZ",m_ViewGroup,false);
                m_cViews[3] = new Checkbox("YZ",m_ViewGroup,false);
                for (int i=0;i<4;i++) {
                    m_cViews[i].addItemListener(this);
                    m_pView.add(m_cViews[i]);
                }
                m_pBnds.add(m_pView);
                
               m_pBnds.add(m_PjViewer.m_clipbounds.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
   //            m_pBnds.add(m_PjViewer.xint.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
//               m_bGetClip = new Button ("Get bounds from current Geom");
//                m_bGetClip.addActionListener(this);
//                m_pBnds.add(m_bGetClip);
                m_bSclAxes = new Checkbox ("Scale Axes to current Bounds");
                m_bSclAxes.addItemListener(this);
                m_pBnds.add(m_bSclAxes);
                
                              
                
                m_tabpanel.addPanel("Graph",m_pMain);
                m_tabpanel.addPanel("Bounds",m_pBnds);
                m_tabpanel.validate();
   
	}
        
        /** Replaces the current graph control panel with the panel associated with the selected graph
         * @param   gname   Name used to reference the graph
         */
        public void changeCP(String gname) {
                cps.removeAll();
                
                PgGeometry newgraph =  (PgGeometry) m_PjViewer.m_graphs.get(gname);
//                cp = newgraph.assureInspector(PsPanel.CONFIG,PsPanel.CONFIG_EXT);
//                 
//                cp.setBorderType(PsPanel.BORDER_NONE);
//                
//                cps.add(cp);
//                
//                update(cp);
//                
//                update(cps);
                cps.add(newgraph.assureInspector(PsPanel.CONFIG,PsPanel.CONFIG_EXT));
                
               
                
                validate();
 //               m_PjViewer.update(m_PjViewer);
                
                
                
        }
        
        /** Handles selections from the pull-down Choice boxes */
        public void itemStateChanged(ItemEvent event) {
		if (m_project==null)
			return;
		Object source = event.getSource();
                if (source == m_selector){
                        changeCP(m_selector.getSelectedItem());
                         return;
                }
                if (source == m_cNew) {
                    int sel = m_cNew.getSelectedIndex();
                    if (sel == 0) { return;}
                    
                    else {
                        String graphname;
                        boolean succ = false;
                        if (m_typetable[sel]==TYPE_VFOS) {
                            if (m_selector.getSelectedIndex() == -1) {return;}
                            String selected = m_selector.getSelectedItem();
                            graphname = "VF on " + selected;
                            if (selected.startsWith("VF")) {
                                succ = false;
                            } else {
                                
                                m_PjViewer.addGraph(m_selector.getSelectedItem(),m_typetable[sel]);
                                succ = true;
                            }
                        } else {
                            graphname = m_typeshortnames[sel] + Integer.toString(m_PjViewer.m_gDef);
                            m_PjViewer.addGraph(graphname, m_typetable[sel]);
                            succ = true;
                        }
                        
                        if (succ) {
                            int res = m_selector.getItemCount();
                            m_selector.add(graphname);
                            m_selector.select(res);
                            changeCP(graphname);
                        }
                        m_cNew.select(0);
                    }
                    return;    
                   
                }
                if (source == m_bSclAxes){
                    if (m_bSclAxes.getState()) {
                        m_PjViewer.setScl();
                        m_PjViewer.fitDisplays();
                        m_PjViewer.update(m_PjViewer);
                    } else {
                        m_PjViewer.unsetScl();
                    }
                }
              
                if (source == m_cClip) {
                    m_PjViewer.setClip(m_cClip.getState());
                }
                if (source == m_cShowFrame) {
                    if (m_cShowFrame.getState()) {
                        PvDisplayIf disp = m_PjViewer.getDisplay();
                        disp.showFrame(true);
                    } else {
                        PvDisplayIf disp = m_PjViewer.getDisplay();
                        disp.showFrame(false);
                    }
                }
                if (source == m_cShowAxes) {
                    PvDisplayIf disp = m_PjViewer.getDisplay();
                    if (m_cShowAxes.getState()) {
                        disp.showAxes(true);
                        disp.showBndBox(true);
                        ((jv.viewer.PvDisplay)disp).getAxes().setMode(PgAxes.AXES_3DBOUNDINGBOX);
                        ((jv.viewer.PvDisplay)disp).getAxes().update( ((jv.viewer.PvDisplay)disp).getAxes());
                        //TODO:  SET axes names
                    } else {
                        disp.showAxes(false);
                        disp.showBndBox(false);
                    }

                }
                for (int i=0; i<4;i++) {
                    if (source == m_cViews[i]) {
                        m_PjViewer.setCamView(i);
                    }
                }
                
        }
        
        /** Handles button presses */
        public void actionPerformed(ActionEvent event) {
		
		Object source = event.getSource();

                if (source == m_bZoomFit) {
                        m_PjViewer.fitDisplays();
                        m_PjViewer.update(m_PjViewer);
                }
                if (source == m_bRemove) {
                    if (m_selector.getItemCount() > 0) {
                         boolean hasvf = m_PjViewer.removeGraph(m_selector.getSelectedItem());
                        
                        if (hasvf) {
                            m_selector.remove("VF on " + m_selector.getSelectedItem());
                        }
                        m_selector.remove(m_selector.getSelectedIndex());
                        
                        if (m_selector.getItemCount()>0){
                            m_selector.select(0);
                            changeCP(m_selector.getSelectedItem());
                        } else {
                             cps.removeAll();
                        }
                    }     
                }
                if (source == m_bGetClip) {
 /*                   PjMulti curgraph = (PjMulti) m_PjViewer.m_graphs.get(m_selector.getSelectedItem());
                    double[] bds = curgraph.getBnds();
                    m_PjViewer.xmin.setValue(bds[0]);
                    m_PjViewer.xmax.setValue(bds[1]);
                    m_PjViewer.ymin.setValue(bds[2]);
                    m_PjViewer.ymax.setValue(bds[3]);
                    m_PjViewer.zmin.setValue(bds[4]);
                    m_PjViewer.zmax.setValue(bds[5]);
                    m_PjViewer.updateClip();
                    m_PjViewer.update(m_PjViewer);
   */             }
                if (source == m_bToClipboard) {
                    PvDisplayIf disp = m_PjViewer.getDisplay();
                    ImageSelection contents = new ImageSelection(disp.getImage());
                    Clipboard systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    systemClipboard.setContents(contents, null);
                    

//                      ImageViewer iviewer = new ImageViewer(disp.getImage());
                }
                
              if (source == m_bAnimstart) {
                    if (m_PjViewer.anim.isRunning()) {
                        m_PjViewer.anim.stop();
                        m_bAnimstart.setLabel("Start");
                    } else {
                        m_PjViewer.anim.start();
                        m_bAnimstart.setLabel("Stop");
                    }
                }
                if ((source == m_tmin)||(source == m_tmax)) {
                    m_PjViewer.anim.setTimeInterval(Double.valueOf(m_tmin.getText()).doubleValue(), Double.valueOf(m_tmax.getText()).doubleValue());
                }
               
	}
        
    public void focusGained(FocusEvent e) {
    }
    
    public void focusLost(FocusEvent event) {
      Object source = event.getSource();
      if ((source == m_tmin)||(source == m_tmax)) {
                    m_PjViewer.anim.setTimeInterval(Double.valueOf(m_tmin.getText()).doubleValue(), Double.valueOf(m_tmax.getText()).doubleValue());
      }
     }
        
        
        
     
        
}
