/*
 * PjPCurve.java
 *
 * Created on December 5, 2004, 5:46 PM
 * Animation added Oct 28, 2005
 */

package Grapher.geom;

import java.awt.Color;

import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.geom.PgPolygon;
import jv.number.PdColor;
import jv.number.PuInteger;
import jv.object.PsDebug;
import Grapher.Viewer.PjViewer;
import jv.object.PsUpdateIf;
import Grapher.util.PuDomain;
import Grapher.util.PuParams;


/**
 * Geometry class for parametrically defined spacecurves
 *
 * @author David Lippman
 * @version 1.0.281005
 * @see PgPolygon
 */
public class PgPCurve extends PgPolygon {
    
//        protected       PgPolygon                       m_curve;
        protected       PuFunction                      m_func2;
        protected       PuDomain                        m_domain;
        protected       boolean                         m_bHasVf = false;
        protected       PjViewer                       m_PjViewer;
        protected       boolean                         m_doanim = false;
        protected       PuParams                        m_params;
    
    /** Creates a new instance of PjPCurve 
     *  @param  PName   Name used to reference graph
     */
    public PgPCurve(String PName) {
        
        if (getClass() == PgPCurve.class)
		  init(PName);
    }
    
    /** Initializes the function and domain and sets up color parameters
     * @param PName Name used to reference graph
     */
    public void init(String PName) {
                    super.init();
                    
                    setName(PName);
                    m_func2 = new PuFunction(1,3);
                    m_func2.setName("Functions");
                    m_func2.setVariables(new String[] {"t"});
                    m_func2.setExpressionNames(new String[] {"x","y","z"});
                    m_func2.setExpressions(new String[] {"cos(t)","sin(t)","t"});
                    m_func2.addParameter("time",1.0);
                    m_func2.setParent(this);
                    
                    m_domain = new PuDomain();
                    m_domain.setVars(new String[] {"t"});
                    m_domain.setDefVals(new double[] {0,2*Math.PI});
                    m_domain.setDefDiscr(new int[] {100});
                    m_domain.init();
                    m_domain.setParent(this);
                    
                    m_params  = new PuParams();
                    m_params.init(3);
                    m_params.registerWith(m_func2);
                    m_params.setParent(this);

                    
                    assureVertexColors();
                    showVertexColors(true);
                    showVertices(false);
                    showEdgeColors(true);
                    setGlobalEdgeSize(4);
      
 	}
    
    /** Does nothing */
    public void start() {
		if (PsDebug.NOTIFY) PsDebug.notify("start: ");
    }
    
    public void setParent(PsUpdateIf parent) {
		m_PjViewer = (PjViewer)parent;
                computeCurve();
    }
 
    /** Handles update calls.  Recomputes graph when domain or function change */
    public boolean update(Object event) {
		if (event == this) {
                        if (m_doanim) {
                            computeCurve();
                        }
                        super.update(this);
                        return true;
		} else if (event == m_domain) {

                        computeCurve();
			return super.update(this);
                } else if (event == m_func2) {
                        if ((m_func2.getExpression(0).indexOf("time")>-1)||(m_func2.getExpression(1).indexOf("time")>-1)||(m_func2.getExpression(2).indexOf("time")>-1)){
                            m_doanim = true;
                        } else {
                            m_doanim = false;
                        }
                        computeCurve();
			return super.update(this);               
                } else {
                
                    // return super.update(event);
                    return super.update(this);
                }
	}
    
    /** Main geometry creation method.  Creates vertices and polygon */
     public synchronized void computeCurve() {
                m_func2.setParameter("time",m_PjViewer.getTime());
                int numVertices = m_domain.getDiscr(0);
                setNumVertices(numVertices);
                double u= m_domain.getMin(0);
                double deltaU = (m_domain.getMax(0)-u)/(numVertices-1.);
                for (int j=0; j<numVertices; j++) {
                 
                                
                                double x,y,z;
                                
                                    x = m_func2.eval(0,u);
                                    y = m_func2.eval(1,u);
                                    z = m_func2.eval(2,u);
                               
                                setVertex(j,x,y,z);
                                int dist	= 255*j/(numVertices-1);
				setVertexColor(j, PdColor.hsv2rgb(dist, 255, 255));		
				u += deltaU;
                                
		}
                makeEdgeFromVertexColors();
     }   
   
    
}
