/*
 * PgVecFldOn.java
 *
 * Created on December 5, 2004, 6:21 PM
 */

package Grapher.geom;

import java.awt.Color;
import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.object.PsPanel;
import jv.geom.PgPointSet;
import jv.number.PdColor;
import jv.number.PuInteger;
import jv.object.PsDebug;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jv.geom.PgVectorField;
import Grapher.util.*;
import jv.object.PsUpdateIf;
import jv.anim.*;
/**
 * Geometry class for creating a vector field attached to another geometry
 *
 * @author David Lippman
 * @version 1.0.122204
 */
public class PgVecFldOn extends PgVectorField implements PsTimeListenerIf {
    protected       PuFunction                      m_func2;
    protected       PuDomain                        m_domain;
    protected       PuParams                        m_params;
    protected       boolean                         m_doanim = false;
    protected       PsAnimation                     m_anim;
    protected       boolean                         m_animset = false;
    
    /** Creates a new instance of PgVecFldOn 
     * @param Pname Name used to reference vectorfield
     */
    public PgVecFldOn(String Pname) {
        super(3, PgVectorField.VERTEX_BASED);
        if (getClass() == PgVecFldOn.class) {
            init(Pname);
        }
    }
    
    public boolean hasAnimation() {
        return m_animset;
    }
    public void setAnimation(PsAnimation anim) {
        m_anim = anim;
        m_animset = true;
    }
    public PsAnimation getAnimation() {
        return m_anim;
    }
    public boolean setTime(PsTimeEvent time) {
        m_func2.setParameter("time",time.getTime());
        return true;
    }
    
    /** Initializes the domain and function
     * @param   Pname   Name used to reference vectorfield
     */
    public void init(String Pname) {
         super.init();
        setName(Pname);
        
        m_func2 = new PuFunction(3,3);
        m_func2.setName("Functions");
        m_func2.setVariables(new String[] {"x","y","z"});
        m_func2.setExpressionNames(new String[] {"dx","dy","dz"});
        m_func2.setExpressions(new String[] {"-y","x","z"});
        m_func2.addParameter("time",1.0);
        m_func2.setParent(this);
        
        m_domain = new PuDomain();
        m_domain.setVars(new String[] {});
        m_domain.setNumDiscr(0);
        m_domain.setIsVF(true);
        m_domain.init();
        m_domain.setParent(this);
        
        m_params  = new PuParams();
        m_params.init(3);
        m_params.registerWith(m_func2);
        m_params.setParent(this);
                    
        assureVectorColors();
        showIndividualMaterial(true);
        showVectorArrows(true);
        setGlobalVectorColor(Color.black);
     
        
       
        
    }
    
    /** Does nothing */
    public void start() {
		if (PsDebug.NOTIFY) PsDebug.notify("start: ");
    }
    

    /** Handles update calls.  Recomputes vectorfield when domain or function changes */
    public boolean update(Object event) {
		if (event == this) {
                      if (m_doanim) {
                            computeVfos();
                        }
			super.update(this);
                        return true;
		} else if (event == m_domain) {
                    computeVfos();
                    return super.update(this);
                    
                } else if (event == m_func2) {
                    if ((m_func2.getExpression(0).indexOf("time")>-1)||(m_func2.getExpression(1).indexOf("time")>-1)||(m_func2.getExpression(2).indexOf("time")>-1)){
                            m_doanim = true;
                        } else {
                            m_doanim = false;
                        }
                    computeVfos();
                    return super.update(this);
                }
                return true;
    }
    
    /** Computes vector field based on vertices in host geometry.  Geometry must be set prior
     * to calling this method.
     */
    public synchronized void computeVfos() {
          
            int numVertices;
            numVertices = getGeometry().getNumVertices();
           
            setNumVectors(numVertices);
            
            int i;
            double x,y,z;
            
          
            for (i=0; i<numVertices;i++) {
                                double[] inp;
                                inp = getGeometry().getVertex(i).getEntries();
      			
                                x = m_func2.eval(0,inp);
                                y = m_func2.eval(1,inp);
                                z = m_func2.eval(2,inp);
                                
                                double scl;
                                if (m_domain.isSclVecs()) {
                                    double length = Math.sqrt(x*x+y*y+z*z);
                                    scl = m_domain.getVeclength()/(3.*length+0.1);
                                } else {
                                    scl = m_domain.getVeclength()/10.;
                                }
                                
                               setVector(i,x*scl,y*scl,z*scl);
                              
                    
            }
      
        }
    
    
    
    
}
